//==============================================================================
// PROJECT      = L_Line - L2 demo
// MODULE       = main.C
// SHORT DESC.  = main module
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
//#include <stdlib.h>
#include "io78f0397.h"
#include "lcd.h"
#include "uart.h"
#include "defines.h"
#include <intrinsics.h>

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Option Byte
//-----------------------------------------------------------------------------
#pragma location = "OPTBYTE"
__root const unsigned char opbyte=0x00;

//---------------------------------------------------------------------------------
// Security ID CODE: for OCD On-Chip Debugging
//---------------------------------------------------------------------------------
#pragma location = "SECUID"
__root const unsigned char secuid[11]={0x02,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff};

//-----------------------------------------------------------------------------
// Global string constants
//-----------------------------------------------------------------------------

const char *startmeldung   =" HF GEN ";
const char *s_clear        ="        ";

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
__saddr volatile unsigned char sw3_in;

__saddr unsigned char menu;

struct    {unsigned char s100;
           unsigned char s10;
           unsigned char s1;}bcd;


//-----------------------------------------------------------------------------
// Module name: Wait
// Description: This module delays the program for (number * 50ms).
//-----------------------------------------------------------------------------
void Wait(unsigned char number)
{
  TMC50 = 0x0b;	        // initialize Timer50
  TCL50 = 0x07;	        // set input clock to fxp / 2^13 = 976 Hz @ 8MHz => 1ms
  CR50  = 0x32;         // set interval time to 50 ms	
  TCE50 = 1;		// start Timer50
  while(number > 0) {
        while(!TMIF50);      // wait for Timer50 Interrupt request flag
        TMIF50 = 0;          // clear Timer50 interruupt request flag
        number--;
        }
  TCE50 = 0;		// stop Timer50
}



long frequenz;
char display[9];

const long steps[8] =
{ 10000000,
   1000000,
    100000,
     10000,
      1000,
       100,
        10,
         1 };

void Frequenz_Out (void)
{
}

void FrequenzToStr (void)
{

/*
  ldiv_t dt;
  long L;
  char x;
  x = 0;
  while (x<8) display[x++] = ' ';
  L = frequenz;
  display[x] = 0;
  while (x)
  { --x;
    dt = ldiv(L,10);
    display[x] = dt.rem | '0';
    L = dt.quot;
    if (L==0) x=0;
  }
*/
}



//-----------------------------------------------------------------------------
// Module:   main
// Description: main module
//-----------------------------------------------------------------------------
void main(void)
{ int xpos;
  char c;

  __disable_interrupt();  // disable all interrupts

  OSCCTL = 0xd0;      // external clock input mode
  PCC = 0x40;         // CPU clock = fxp
  MCM = 0x06;         // Set main system clock and peripheral hardware clock
  OSTS = 0x00;        // Set minimum oscillation stabilization time

  PU7 = 0x1F;         // enable Pull-up resistor
  KRM = 0x1F;         // enable key interrupt detection
  KRMK = 0;           // enable key interrupt

  WTM = 0x83;         // enable watch timer -> interval timer = 0.5 sec
  WTMK = 0;           // enable watch timer interrupt

  __enable_interrupt();   // enable all interrupts

  LCD_init();         // Initialization of LCD controller
  UART_init();        // Initialization of UART6 controller
  frequenz = 1000000; // default 1 MHz

//  LCD_string(&startmeldung[0],0);
//  Wait(100);


  while(!sw3_in)
  {
    LCD_string_shift("NEC L_LINE");
    LCD_string_shift("L2 DEMOKIT");
    LCD_string_shift("PRESS TASTE TO START");
  }
  sw3_in = 0;

  sw3_in = 0;
  xpos = 0;

  Frequenz_Out();
  FrequenzToStr();



  LCD_string(&display[0],0);

immerzu:
    while(!sw3_in);
    c = 0;
    switch (sw3_in)
    { case UP:    frequenz += steps[xpos]; c=1; break;
      case DOWN:  frequenz += steps[xpos]; c=1; break;
      case RIGHT: ++xpos; break;
      case LEFT: --xpos; break;
      case SELECT: break;
    }
    if (frequenz < 0) frequenz = 0;
    if (frequenz > 40000000) frequenz = 40000000;
    if (xpos > 7) xpos = 7;
    if (c)
    {

      c = 0;
    }
    Wait(4);
    sw3_in = 0;
    LCD_string(&display[0],0);
    sw3_in = 0;
    goto immerzu;
}



//-----------------------------------------------------------------------------
// ISR:      INTKR_vect
// Function: Key interrupt service routine / navigator switch / key debouncing
//-----------------------------------------------------------------------------
#pragma vector = INTKR_vect
__interrupt void isr_INTKR(void)
{
    unsigned char sw3_first,sw3_second;
    sw3_first= (~P7) & 0x1f;     // read SW3 first time
    TMC51 = 0x0b;	        // initialize Timer51
    TCL51 = 0x07;	        // set input clock to fxp / 2^12 = 1953 Hz @ 8MHz => 0,5ms
    CR51  = 0x64;               // set interval time to 5 ms	
    TCE51 = 1;	     	        // start Timer51
    while(!TMIF51);             // wait for Timer51 Interrupt
    TMIF51=0;
    TCE51 = 0;	     	        // stop Timer51
    sw3_second= (~P7) & 0x1f;     // read SW3 second time
    if(sw3_first==sw3_second) sw3_in=sw3_first; // debounce SW3
    else sw3_in=0;
}
