//==============================================================================
// PROJECT      = L_Line - L2 demo
// MODULE       = temp.c
// SHORT DESC.  = implementation of temperature measurement via dual slop methode
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "io78f0397.h"
#include "lcd.h"
#include "uart.h"
#include "defines.h"
#include <intrinsics.h>

//-----------------------------------------------------------------------------
// Function prototyps
//-----------------------------------------------------------------------------
extern void Wait(unsigned char Number);
extern __callt void  BCD_calc(unsigned char data);


//-----------------------------------------------------------------------------
// External variables
//-----------------------------------------------------------------------------

extern __saddr volatile unsigned char sw3_in;


extern struct    {unsigned char s100;
       		  unsigned char s10;
                  unsigned char s1;}bcd;

//-----------------------------------------------------------------------------
// Global constants
//-----------------------------------------------------------------------------
extern const char *degree_cel;
extern const char *degree_fah;


// resistor values -20C -> 0C
const short temperature_neg[21]={1341,1353,1366,1378,1390,1402,1415,1427,1440,1453,
                                 1465,1478,1491,1504,1517,1530,1544,1557,1570,1584,
                                 1597};
// resistor values  0C -> +71C
const short temperature_pos[72]={1597,1611,1624,1638,1652,1666,1680,1694,1708,1722,
                                 1736,1751,1765,1780,1794,1809,1824,1838,1853,1868,
                                 1883,1889,1914,1929,1944,1960,1975,1991,2006,2022,
                                 2038,2054,2069,2085,2102,2118,2134,2150,2167,2183,
                                 2200,2216,2233,2250,2267,2284,2301,2318,2335,2352,
                                 2369,2387,2404,2422,2439,2457,2475,2493,2511,2529,
                                 2547,2565,2583,2601,2620,2638,2657,2675,2694,2713,
                                 2731,2750};

//-----------------------------------------------------------------------------
// Global variables
//-----------------------------------------------------------------------------
volatile __saddr unsigned char DS_Ready, DS_OVF_error, sendokflag;

__saddr char tmpmode;

__saddr unsigned long ref_time, meas_time;
__saddr unsigned long calc_value;
__saddr unsigned char ds_status;

__saddr unsigned long avr_meas_time;
__saddr unsigned long avr_ref_time;

//-----------------------------------------------------------------------------
// Module name: discharge_C
// Description: discharge of capacitor to prepare new measurement
//-----------------------------------------------------------------------------
__callt void discharge_C (void)
{
  unsigned int i;
  PM0 &= 0xF2; // switch all ports to output
  P0 &= 0xF2; // discharge C
  for (i=0;i<2000;i++) // wait for discharging the capacitor
  {
      __no_operation();
  }
}

//-----------------------------------------------------------------------------
// Module name: temperature
// Description: temperature measurement using dual slope methode
//-----------------------------------------------------------------------------
void temp()
{
  unsigned char avr_count;
  unsigned char tpos;
  unsigned char i;

  PM0 = 0x00;
  PU0 = 0x00;

  //16-bit timer setting
  TMC00 = 0x00; // operation stop
  PRM00 = 0x10; // clock frequency 8 MHz

  // TI000 rising edge
  CRC00 = 0x04; // CR010 operates as capture register
  TOC00 = 0x00; // no timer-output
  TMIF010 = 0; // reset interruptflag

  ds_status = 0; // initialize dual-slope flags and status counter
  DS_Ready = 0;
  DS_OVF_error = 0;
  TMMK010 = 0; // start conversion by enabling the timer
  TMIF010 = 1; // interrupt and setting the interrupt flag

  avr_meas_time = 0;
  avr_ref_time  = 0;

  while(sw3_in!=LEFT)
  {
     if (DS_Ready)
     {
        if (!DS_OVF_error)
        {
           avr_meas_time += meas_time;
           avr_ref_time  += ref_time;
           avr_count++;
           if(avr_count==100)
           {
             avr_meas_time/=100;
             avr_ref_time/=100;
             calc_value=(avr_meas_time * 10000)/ avr_ref_time;// calculate resistor value of
             calc_value-=10000;                               // temperature sensor KTY 13-5

             if(calc_value<1597)  // temperature is negative
             { tpos=0;
               for(i=0;i<20;i++)
                {
                  if((calc_value>=temperature_neg[i])&&(calc_value<temperature_neg[i+1])){ i=20-i; break;}
                }
             }
             else                 // temperature is positive
             {  tpos=1;
                for(i=0;i<=70;i++)
                {
                    if((calc_value>=temperature_pos[i])&&(calc_value<temperature_pos[i+1])) break;
                }
             }
             switch(sw3_in)
             {
                   case UP:
                   case DOWN: tmpmode=!tmpmode;
                               break;
             }
             sw3_in=0;

             if(!tmpmode) //display temperature in degree celsius
             {
               BCD_calc(i);
               if(tpos) LCD_putc(0,0xfb);
               else LCD_putc(0,0xfd);
               LCD_putc (1,0xf0);
               LCD_putc (2, bcd.s10);
               LCD_putc (3, bcd.s1);
               LCD_string(&degree_cel[0],5);
              }
              else       //display temperature in degree fahrenheit
              {
                if(!tpos)   // negative value in degree fahrenheit
                {
                  switch(i)
                  {
                      case 18: i=0; tpos=1; break;
                      case 19: i=2; tpos=0; break;
                      case 20: i=4; tpos=0; break;
                      default: i=32-i*9/5; tpos=1; break;
                  }
                 }
                 else       // positive value in degree fahrenheit
                 {
                      i=i*9/5+32; tpos=1;
                 }
                 if(tpos) LCD_putc(0,0xfb);
                 else LCD_putc(0,0xfd);
                 BCD_calc(i);
                 if(bcd.s100) LCD_putc(1,bcd.s100);
                 else LCD_putc(1,0xf0);
                 LCD_putc (2, bcd.s10);
                 LCD_putc (3, bcd.s1);
                 LCD_string(&degree_fah[0],5);
              }

              if(tpos) UART_SendString("+ ");
              else UART_SendString("- ");
              if(bcd.s100) UART_SendChar(bcd.s100+0x30);
              UART_SendChar(bcd.s10+0x30);
              UART_SendChar(bcd.s1+0x30);
              if(!tmpmode) UART_SendString(" degree celsius     \r");
              else UART_SendString(" degree fahrenheit \r");

              avr_count=0;
              avr_meas_time = 0;
              avr_ref_time  = 0;
           }
        }
        ds_status = 0;        // initialize dual-slope flags and status-counter
        DS_Ready = 0;
        DS_OVF_error = 0;
        TMMK010 = 0;          // start new conversion
        TMIF010 = 1;
      }
   }
  sw3_in=0;
}

//-----------------------------------------------------------------------------
// ISR: 	  isr_INTTM010
// Function: Interrupt service routine of Timer010 for Dual Slope measurement
//-----------------------------------------------------------------------------
#pragma vector=INTTM010_vect
__interrupt void isr_INTTM010(void)
{
  unsigned int CR_value;
  CR_value = CR010;
  TMC00 = 0x00; // stop and clear timer
  switch(ds_status)
  {
    case 0:                   // start of dualslope conversion with
            discharge_C();    // discharging the C
            PM0 |= 0x09;
            P0_bit.no2 = 1;   // set var. resistor port to high level
            TMC00 = 0x06;     // start free running-mode
            ds_status++;
            break;
    case 1:                   // start of dualslope 2. conversion with discharging the C
            if (OVF00)        // if overflow flag is set measurement is invalid
            {
              DS_Ready=1;
              DS_OVF_error=1;
              ds_status=0;
            }
            else
            {
              ref_time = CR_value;
              discharge_C();
              PM0 |= 0x05;
              P0_bit.no3 = 1; // set reference resistor port to high level
              TMC00 = 0x06;   // start free running-mode
              ds_status++;
            }
            break;
    case 2:                   // end of dualslope conversion calculating value
            if (OVF00)        // if overflow flag is set measurement is invalid
            {
              DS_OVF_error=1;
            }
            else
            {
              meas_time = CR_value;
              PM0 &= 0xFE;    // enable discharge
              P0 &= 0xFE;
              TMMK010 = 1;    // conversion disabled
            }
            ds_status=0;      // reset dual slope status
            DS_Ready=1;       // conversion finished
            break;
    }
}

