//==============================================================================
// PROJECT      = L_Line - L2 demo
// MODULE       = uart.c
// SHORT DESC.  = UART driver routines
// DEVICE       = 78K0/LG2
// VERSION      = 1.0
// DATE         = 01.09.2005
// LAST CHANGE  =
// =============================================================================
// By:          NEC Electronics (Europe) GmbH
//              Arcadiastrasse 10
//              D-40472 Duesseldorf
// =============================================================================

//-----------------------------------------------------------------------------
// Include files
//-----------------------------------------------------------------------------
#include "io78f0397.h"
#include "defines.h"
#include <intrinsics.h>

//-----------------------------------------------------------------------------
// Module:   init_UART
// Function: Initialization of UART6
//-----------------------------------------------------------------------------
void UART_init(void)
{

  PM1_bit.no4=1;            //input mode for RxD
  PM1_bit.no3=0;            //output mode for TxD
  P1_bit.no3=1;             //set TxD output to high level

#if BAUDRATE==9600          //initialization for baudrate = 9600
  CKSR6 = 0x02;
  BRGC6 = 104;
#elif BAUDRATE==38400       //initialization for baudrate = 38400
  CKSR6 = 0x00;
  BRGC6 = 104;
#elif BAUDRATE==115200      //initialization for baudrate = 115200
  CKSR6 = 0x00;
  BRGC6 = 35;
#endif

  POWER6 = 1;               //enable internal clock operation

  ASIM6 |= 0xE5;            //enable transmission
                            //enable reception
                            //no parity
                            //character lenght of data = 8-bits
                            //number of stop bits = 1
}

//-----------------------------------------------------------------------------
// Module:   UART_SendChar
// Function: Send char via UART6
//-----------------------------------------------------------------------------
void UART_SendChar (unsigned char ucData)
{
    TXB6 = ucData;              // load UART transmit buffer
    while(!STIF6);              // wait for transmission finished
    STIF6=0;
}


//-----------------------------------------------------------------------------
// Module:   UART_GetChar
// Function: Get char from UART6
//-----------------------------------------------------------------------------
unsigned char UART_GetChar(void)
{
     unsigned char receive_byte;
     while(!SRIF6);               // wait for uart receive byte
     receive_byte = RXB6;         // load UART receive buffer
     SRIF6=0;
     return (receive_byte);
}

//-----------------------------------------------------------------------------
// Module:   UART_SendString
// Function: Send string via UART6
//-----------------------------------------------------------------------------
void UART_SendString (char *ucpStr)
{
    unsigned char ucData;
    while (1)
    {
       	ucData = (unsigned char)(*ucpStr++);	
	if(ucData) UART_SendChar (ucData);	
	else break;
    }
}

